(function() {
  'use strict';

  angular.module('abstractpapers')
    .factory('$abstractpapers', ['$common', '$database', function($common, $database) {
      var Abstractpaper = $common.model();
      Abstractpaper.tableName = 'abstractpapers';
      Abstractpaper.fields = {
        'id': 'text primary key',
        'title': 'text',
        'authors': 'text',
        'institutes': 'text',
        'text': 'text',
        'external_link': 'text',
        'order': 'integer',
        'head': 'text',
        'embargo': 'date'
      };

      Abstractpaper.list = function() {
        var query = 'SELECT abstractpapers.id, abstractpapers.\'order\', abstractpapers.title, schedualassignments.code, roles.function, group_concat(persons.last_name) AS speaker FROM abstractpapers JOIN presentations ON presentations.abstracts = abstractpapers.id JOIN schedualassignments ON schedualassignments.presentation = presentations.id JOIN roles ON roles.chair = presentations.id JOIN persons ON persons.id = roles.person  WHERE schedualassignments.code IS NOT NULL AND  (abstractpapers.embargo IS NULL OR abstractpapers.embargo < date(\'now\')) GROUP BY abstractpapers.id ORDER BY roles.\'order\' ';
        return $database.getItems(query, []);
      };

      Abstractpaper.prototype.getPresentations = function() {
        var query = 'SELECT schedualassignments.code, schedualassignments.start_time, schedualassignments.end_time, presentations.id,sessions.color, presentations.abstracts, presentations.title, presentations.outline, presentations.external_link, persons.last_name, persons.first_name, persons.country  FROM schedualassignments JOIN presentations ON schedualassignments.presentation = presentations.id JOIN roles ON roles.chair = presentations.id JOIN persons ON persons.id = roles.person JOIN sessions on schedualassignments.session = sessions.id WHERE presentations.abstracts = ? GROUP BY presentations.id ORDER BY schedualassignments.start_time, roles.\'order\'';
        return $database.getItems(query, [this.id]);
      };

      return Abstractpaper;
    }]);
})();
