// Das chatmodul ist eine Lokale Betaversion eines "Chats" der auch in sessions etc als diskussionsforum genutzt werden kann...
// Zu demozwecken ist der Chat in den sessions eingebunden

'use strict';

/**
 * Ein chatfenster öffnet sich automatisch wenn man die Route anspricht.
 *
 * @param title Der Titel des Fensters.
 * @param entity_id Eine unique Id, die für dieses Fenster Benutzt wird.
 */

angular.module('chat', ['chat.controller', 'chat.model']).config(function($stateProvider) {
  $stateProvider.state('app.chatid', {
    url: '/chat/:name/:id',
    views: {
      'menuContent': {
        templateUrl: 'modules/chat/views/chat.html',
        controller: 'ChatCtrl'
      }
    }
  }).state('app.chat', {
    url: '/chat',
    views: {
      'menuContent': {
        templateUrl: 'modules/chat/views/chat-list.html',
        controller: 'ChatListCtrl',

      }
    },
    loggedIn: true,
    chat: true
  }).state('app.chat-deactivate', {
    url: '/chat/deactivated',
    views: {
      'menuContent': {
        templateUrl: 'modules/chat/views/deactivated.html',
        controller: 'ChatActivateCtrl'
      }
    }
  });

}).run(function(SocketProvider, $pubsub, Messages, User, $location, $rootScope, $timeout) {
  var registerStateChangeListener = function() {
    $rootScope.$on('$stateChangeStart', function(event, toState) {
      var chat = toState.chat;
      if (User.secret && chat && !User.chat) {
        $location.path('/app/chat/deactivated', {});
        return false;
      }
    });
  };

  var registerSocketListener = function() {
    SocketProvider.on('private message', function(msg) {
      if (msg) {
        var message = new Messages(msg);
        message.save();
        if (User.public !== message.sender) {
          $pubsub.publish('#/app/chat', function(scope) {
            scope.count = scope.count + 1;
            scope.$apply();
          });
        }
      }
    });
  };

  var publishUnreadMessages = function() {
    // timeout is needed to ensure, that the menu item is already loaded.
    $timeout(function() {
      Messages.unreadMessages().then(function(unread) {
        $pubsub.publish('#/app/chat', function(scope) {
          scope.count = unread.count;
          scope.$apply();
        });
      });
    }, 1500);
  };

  if (User.chat) {
    registerSocketListener();
    registerStateChangeListener();
    publishUnreadMessages();
  }



});
