'use strict';

angular.module('chat.controller', []).controller('ChatCtrl', function($scope, $stateParams, $ionicScrollDelegate, $timeout, $filter, User, Messages, Socket, $chatcontacts) {
  $scope.me = User;
  $scope.me.ini = 'Me';
  var name = $stateParams.name;
  $scope.name = name;
  var id = $stateParams.id;

  $scope.contact = {
    ini: name.substring(0, 1),
    id: id
  };

  $scope.messages = [];
  Messages.forContact($scope.contact.id).then(function(messages) {
    $chatcontacts.markAsChecked(id, name, messages[messages.length - 1].created_at);
    messages.forEach(function(msg) {
      displayMessage(msg);
    });
  });

  $scope.sendMessage = function() {
    if ($scope.messageText) {
      var msg = {
        recipient: $stateParams.id,
        message: $scope.messageText
      };

      Socket.emit('private message', msg);
      $scope.messageText = '';
    }
  };

  var displayMessage = function(message) {
    message.side = message.sender === User.public ? 'right' : 'left';
    message.ini = message.sender === User.public ? $scope.me.ini : $scope.contact.ini;
    $scope.messages.push(message);
    $scope.$apply();
    $scope.scrollBottom();
  };

  var receiveMessage = function(message) {
    displayMessage(message);
    $chatcontacts.markAsChecked(id, name, message.created_at);
  };

  $scope.$on('$viewContentLoaded', function() {
    Socket.on('private message', receiveMessage);
  });

  $scope.$on('$destroy', function() {
    Socket.removeListener('private message', receiveMessage);
    $scope.name = '';
  });

  $scope.scrollBottom = function() {
    $ionicScrollDelegate.$getByHandle('messages').scrollBottom(true);
  };

  $scope.scrollUp = function() {
    $ionicScrollDelegate.$getByHandle('messages').scrollTop(true);
  };

  $timeout(function() {
    $scope.scrollBottom();
  }, 300);

}).controller('ChatListCtrl', function($scope, $persons, $filter, $chatcontacts, Socket) {

  $scope.edit = false;
  $scope.search = '';
  $scope.items = [];

  $scope.addPerson = function(person) {
    $chatcontacts.addPerson(person.id, person.name);
    $scope.items.push(person);
  };

  $scope.limitResults = function() {
    if ($scope.search.length > 0) {
      return 3;
    }
    return 0;
  };

  $scope.search = function() {
    var searchValue = $scope.searchValue;
    if (searchValue.length > 0) {
      $chatcontacts.search(searchValue)
        .success(function(result) {
          $scope.persons = result;
        });
    } else {
      $scope.persons = [];
    }
  };

  $chatcontacts.all().then(function(contacts) {
    $scope.items = contacts;
  });

  $scope.clearSearch = function() {
    $scope.searchValue = '';
  };

  $scope.editMode = function() {
    $scope.edit = !$scope.edit;
  };

  $scope.headTitle = 'Contacts';

  var receiveMessage = function() {
    $chatcontacts.all().then(function(contacts) {
      $scope.items = contacts;
    });
  };

  $scope.$on('$viewContentLoaded', function() {
    Socket.on('private message', receiveMessage);
  });

  $scope.$on('$destroy', function() {
    Socket.removeListener('private message', receiveMessage);
  });
}).controller('ChatActivateCtrl', function($scope, User, $location) {
  $scope.activateChat = function() {
    User.chat = true;
    User.save();
    $location.path('/app/chat');
  };
});
