'use strict';

angular.module('chat.model', []).factory('Messages', function($common, $database) {
  var Messages = $common.model();
  Messages.tableName = 'messages';
  Messages.fields = {
    'id': 'text primary key',
    'sender': 'text',
    'recipient': 'text',
    'message': 'text',
    'created_at': 'date'
  };

  Messages.forContact = function(id) {
    var query = 'SELECT * FROM messages WHERE sender = ? OR recipient = ?';
    return $database.getItems(query, [id, id]);
  };

  Messages.unreadMessages = function() {
    var query = 'SELECT COUNT(messages.id) AS count FROM messages JOIN chatcontacts ON messages.recipient = chatcontacts.id WHERE messages.created_at > chatcontacts.last_checked';
    return $database.getItem(query, []);
  };

  return Messages;
}).factory('$chatcontacts', function($common, $database, User, $http, $settings) {
  var Chatcontacts = $common.model();
  Chatcontacts.tableName = 'chatcontacts';
  Chatcontacts.fields = {
    'id': 'text primary key',
    'name': 'text',
    'last_checked': 'date'
  };

  Chatcontacts.all = function() {
    var query = 'SELECT chatcontacts.id, chatcontacts.name AS name, messages.message AS last_message, SUM(case when messages.created_at > chatcontacts.last_checked then 1 else 0 end) as unread_messages, MAX(messages.created_at) AS created_at, chatcontacts.last_checked FROM chatcontacts LEFT JOIN messages ON chatcontacts.id = messages.sender OR chatcontacts.id = messages.recipient WHERE chatcontacts.id IS NOT ? AND (last_message IS NOT NULL OR last_checked IS NOT NULL) GROUP BY chatcontacts.name ORDER BY messages.created_at DESC';
    return $database.getItems(query, [User.public], Chatcontacts);
  };

  Chatcontacts.addPerson = function(id, name) {
    var query = 'INSERT INTO chatcontacts (id, name, last_checked) VALUES (?,?,?)';
    return $database.execute(query, [id, name, '0']);
  };

  Chatcontacts.markAsChecked = function(id, name, datetime) {
    var query = 'INSERT OR REPLACE INTO chatcontacts (id, name, last_checked) VALUES (?, ?, ?)';
    return $database.execute(query, [id, name, datetime]);
  };

  Chatcontacts.search = function(search) {
    var url = $settings.apiHost + '/conference/' + $settings.conferenceId + '/chat/search/' + search;
    return $http.get(url);
  };

  return Chatcontacts;
});
