'use strict';
angular.module('common.basefactory', [])
.factory('$basefactory', function($database) {
  var basefactory = {};

  basefactory.getBasefactory = function(modelClass) {

    var buildBaseQuery = function(options) {
      options = options || {};
      var selectColumns = {},
      joinStatements  = [],
      selectStatements = [],
      buildJoinStatement = function(relationField) {
        var relatedColumn = relationField.relatedColumn || 'id',
        baseColumn = relationField.field;

        selectColumns[baseColumn] = relationField.tableName;
        return 'LEFT JOIN ' + relationField.tableName + ' ON ' + modelClass.tableName + '.' + baseColumn + ' = ' + relationField.tableName + '.' + relatedColumn;
      };

      for(var field in modelClass.fields) {
        selectColumns[field] = modelClass.tableName;
      }

      if(options.selectRelated) {
        options.selectRelated.forEach(function(fieldName) {
          var relationField = modelClass.relatedFields[fieldName];
          joinStatements.push(buildJoinStatement(relationField));
        });
      }

      for(var fieldName in selectColumns) {
        selectStatements.push('"' + selectColumns[fieldName] + '"."' + fieldName + '"');
      }

      return 'SELECT ' + selectStatements.join(', ') + ' FROM ' + modelClass.tableName + ' ' + joinStatements.join(' ');
    };

    var getMultipleItems = function(ids, options) {
      var query = buildBaseQuery(options);
      query += ' WHERE id IN (';
      var params = [];
      ids.forEach(function() { params.push('?');});
      query += params.join(',') + ')';
      return $database.getItems(query, ids);
    };

    return {
      get: function(id, options) {
        if (Array.isArray(id)) {
          return getMultipleItems(id, options);
        }
        var query = buildBaseQuery(options);
        query += ' WHERE ' + modelClass.tableName + '.id = ?';
        return $database.getItem(query, [id]);
      },
      all: function(options) {
        var query = buildBaseQuery(options);
        return $database.getItems(query, []);
      },
      forIds: getMultipleItems,
      model : modelClass
    };
  };

  return basefactory;
});