'use strict';

angular.module('common.model', []).factory('$commonModel', function($database) {
	var basemodel = function() {
		function Model(values) {

			// CONSTRUCTOR
			for (var key in values) {
				this[key] = values[key];
			}

			// CREATE

			this.save = function() {
				var insertStatement = this.hasOwnProperty('insertStatement') ? this.insertStatement : buildInsertStatement(this), insertValues = this.hasOwnProperty('insertValues') ? this.insertValues : buildInsertValues(this);
				return $database.execute(insertStatement, insertValues);
			};

			function buildInsertValues(self) {
				var values = [];
				for (var field in Model.fields) {
					if (self.hasOwnProperty(field)) {
						var value = self[field];
						// This validation is needed, otherwise incomming arrays will be saved as JSON-Arrys, which isn't necessarily a bad thing, but it broke some other functions.
						if (value !== null && Model.fields[field] === 'text') {
							value = value.toString();
						}
						values.push(value);
					}
				}
				return values;
			}

			function buildInsertStatement(self) {
				var query = 'INSERT OR REPLACE INTO ';
				var fields = [];
				var params = [];

				for (var field in Model.fields) {
					if (self.hasOwnProperty(field)) {
						fields.push('"' + field + '"');
						params.push('?');
					}
				}

				query += Model.tableName + ' (' + fields.join(',') + ') VALUES(' + params.join(',') + ')';
				return query;
			}

			// DELETE

			this.delete = function() {
				var query = 'DELETE FROM ' + Model.tableName + ' WHERE id = ?';
				$database.execute(query, [this.id]);
			};

		}

		// READ

		Model.get = function(id, options) {
			if (Array.isArray(id)) {
				return getMultipleItems(id, options, Model);
			} else {
				return getSingleItem(id, options, Model);
			}
		};

		Model.all = function(options) {
			var query = buildSelectStatement(options);
			return $database.getItems(query, [], Model);
		};

		Model.find = function(searchString, options) {
			var query = buildSelectStatement(options);
			for (var searchField in Model.searchFields) {
				query += ' WHERE ' + Model.tableName + '.' + searchField + ' = "%' + searchString + '%"';
			}
			return $database.getItems(query, [], Model);

		};

		function getSingleItem(id, options, Model) {
			var query = buildSelectStatement(options);
			query += ' WHERE ' + Model.tableName + '.id = ?';
			return $database.getItem(query, [id], Model);
		}

		function getMultipleItems(ids, options) {
			var query = buildSelectStatement(options);
			query += ' WHERE id IN (';
			var params = [];
			ids.forEach(function() {
				params.push('?');
			});
			query += params.join(',') + ')';
			return $database.getItems(query, ids, Model);
		}

		function buildSelectStatement(options) {
			var selectColumns = {}, joinStatements = [], selectStatements = [];

			options = options || {};

			for (var field in Model.fields) {
				selectColumns[field] = Model.tableName;
			}

			if (options.selectRelated) {
				options.selectRelated.forEach(function(fieldName) {
					var relationField = Model.relatedFields[fieldName];
					joinStatements.push(buildJoinStatement(relationField));
				});
			}

			for (var fieldName in selectColumns) {
				selectStatements.push('"' + selectColumns[fieldName] + '"."' + fieldName + '"');
			}

			function buildJoinStatement(relationField) {
				var relatedColumn = relationField.relatedColumn || 'id', baseColumn = relationField.field;

				selectColumns[baseColumn] = relationField.tableName;
				return 'LEFT JOIN ' + relationField.tableName + ' ON ' + Model.tableName + '.' + baseColumn + ' = ' + relationField.tableName + '.' + relatedColumn;
			}

			return 'SELECT ' + selectStatements.join(', ') + ' FROM ' + Model.tableName + ' ' + joinStatements.join(' ');
		}

		return Model;
	};

	return basemodel;
});
