'use strict';

angular.module('database', ['ionic', 'ngCordova', 'database.sqlite', 'database.websql']).factory('$database', function($http, $settings, $websqlDriver, $sqliteDriver, $cordovaProgress, $q, $pubsub) {

  var databaseDriver = ionic.Platform.isWebView() ? $sqliteDriver : $websqlDriver;

  var buildInsertStatement = function(Model) {
    var query = 'INSERT OR REPLACE INTO ' + Model.tableName;
    var fields = [];
    var params = [];

    for (var field in Model.fields) {
      fields.push('"' + field + '"');
      params.push('?');
    }
    query += ' (';
    query += fields.join(',');
    query += ') VALUES ';
    // The query should now look like this:
    //query = 'INSERT OR REPLACE INTO "tableName" (field1, field2, field3...) VALUES

    query += '(' + params.join(',') + ')';

    return query;
  };

  var buildDeleteStatement = function(tablename, objs) {
    var ids = objs.map(function(obj) {
      return '\'' + obj.id + '\'';
    }).join(',');
    var query = 'DELETE FROM ' + tablename + ' WHERE id IN (' + ids + ')';
    return query;
  };
  var buildValueArray = function(Model, data) {
    var values = [];
    for (var field in Model.fields) {
      values.push(data[field]);
    }
    return values;
  };

  var saveTimestamp = function(modelClass) {
    $http.get($settings.apiHost + '/timestamp').then(function(result) {
      var timestamp = result.data;
      databaseDriver.execute('INSERT INTO timestamps (tablename, timestamp) VALUES (?, ?)', [modelClass.tableName, timestamp]);
    });
  };

  return {
    getDB: databaseDriver.getDB,

    execute: databaseDriver.execute,

    massExecute: databaseDriver.massExecute,

    getItem: function(preparedStatement, parameters, Model) {
      return this.execute(preparedStatement, parameters).then(function(result) {
        if (result.rows.length > 0) {
          return Model ? new Model(result.rows.item(0)) : result.rows.item(0);
        } else {
          return null;
        }
      }, function(error) {
        return error;
      });
    },

    getItems: function(preparedStatement, parameters, Model) {
      return this.execute(preparedStatement, parameters).then(function(res) {
        var items = [];
        for (var i = 0; i < res.rows.length; i++) {
          var item = Model ? new Model(res.rows.item(i)) : res.rows.item(i);
          items.push(item);
        }
        return items;
      }, function(err) {
        return err;
      });
    },

    createTableForModel: function(model) {
      var query = 'CREATE TABLE IF NOT EXISTS ';
      query += model.tableName;
      query += '(';
      var fields = [];
      for (var key in model.fields) {
        fields.push('"' + key + '"' + ' ' + model.fields[key]);
      }
      query += fields.join(',');
      query += ')';
      return this.execute(query, []);
    },
    save: function(model, data) {
      var operations = [];
      var deleteObjects = [];
      data = data.filter(function(obj) {
        if (obj.deleted_at) {
          deleteObjects.push(obj);
          return false;
        } else {
          return true;
        }
      });

      data.forEach(function(item) {
        var sql = {};
        sql.query = buildInsertStatement(model);
        sql.params = buildValueArray(model, item);
        operations.push(sql);
      });

      this.execute(buildDeleteStatement(model.tableName, deleteObjects));
      return this.massExecute(operations);
    },

    syncResource: function(modelClass, url) {
      var database = this;
      this.getItem('SELECT * FROM timestamps WHERE tablename = ? ORDER BY id DESC LIMIT 1', [modelClass.tableName]).then(function(timestamp) {
        url += (timestamp === null) ? '' : '?timestamp=' + timestamp.timestamp;
        $http.get(url).success(function(items) {
          database.save(modelClass, items).then(function() {
            saveTimestamp(modelClass);
            $pubsub.publish(modelClass.tableName, 'updated');
            $pubsub.publish('updated', modelClass.tableName);
          });
        });
      });
    },
  };
});
