'use strict';

angular.module('database.sqlite', []).factory('$sqliteDriver', function($cordovaSQLite, $q, $window, $ionicPlatform, mailbox) {
  var _db;

  var getDB = function() {
    if (!_db && ionic.Platform.isReady) {
      _db = $window.sqlitePlugin.openDatabase({
        name: 'kongressApp2.db',
        location: 2,
        bgType: 0,
        createFromLocation: 1
      });
    }
    return _db;
  };

  if(ionic.Platform.isWebView()) {
    $ionicPlatform.ready(function() {
      var db = getDB();
      while(mailbox.hasNext('database')) {
        var operation = mailbox.next('database');
        $cordovaSQLite.execute(db, operation.query, operation.params).then(resolvePromise(operation.defer));
      }
    });
  }

  var resolvePromise = function(promise) {
    return function(data) {
      promise.resolve(data);
    };
  } ;

  return {
    getDB: getDB,
    execute: function(query, params) {
      var db = this.getDB();
      if(db) {
        return $cordovaSQLite.execute(db, query, params);
      } else {
        var defer = $q.defer();
        mailbox.write('database', {query: query, params: params, defer: defer});
        return defer.promise;
      }

    },
    massExecute: function(operations) {
      var db = this.getDB();
      var transactionPromise = $q.defer();
      var rowPromises = [];
      db.transaction(function(transaction) {
        operations.forEach(function(operation) {
          var rowPromise = $q.defer();
          rowPromises.push(rowPromise);
          transaction.executeSql(operation.query, operation.params, function(transaction, result) {
            rowPromise.resolve(result);
          });
        });
        $q.all(rowPromises).then(function() {
          transactionPromise.resolve('done');
        });
      });
      return transactionPromise.promise;
    }
  };
});
