(function() {
  'use strict';

  angular.module('exhibitions').controller('ExhibitionListCtrl', ['$scope', '$exhibitions', '$filter', '$translate',
  function($scope, $exhibitions, $filter, $translate) {
    $scope.isAndroid = ionic.Platform.isAndroid();
    var letters = $scope.letters = [];
    var exhibs = $scope.exhibs = [];
    var currentCharCode = 'A'.charCodeAt(0) - 1;
    $translate('SEARCH').then(function(item) {
      $scope.placeholder = item;
    });

    $scope.topExhibitors = [];

    function addLetter(code) {
      var letter = String.fromCharCode(code);
      exhibs.push({
        isLetter : true,
        letter : letter
      });
      letters.push(letter);
    }


    $exhibitions.all().then(function(exhibitions) {
      $scope.exhibitions = $filter('orderBy')(exhibitions, 'name');

      $exhibitions.getTopExhibitors().then(function(topexhibitors) {
        topexhibitors.forEach(function(exhib) {

          if (exhib.locations) {
            exhib.getLocations().then(function(locations) {
              exhib.location = '';
              locations.forEach(function(location) {
                exhib.location += location.name + ' ';
              });
            });
          }
          //toplist.push(exhib);

        });
        $scope.topExhibitors = topexhibitors;
        var letter = 'Aussteller';
        $scope.topExhibitors.push({
          isLetter : true,
          letter : letter
        });

      });

      $scope.exhibitions.forEach(function(exhibition) {
        //Get the first letter of the last name, and if the last name changes
        //put the letter in the array
        var exhibitionCharCode = exhibition.name.toUpperCase().charCodeAt(0);
        //We may jump two letters, be sure to put both in
        //(eg if we jump from Adam Bradley to Bob Doe, add both C and D)
        var difference = exhibitionCharCode - currentCharCode;
        for (var i = 1; i <= difference; i++) {
          addLetter(currentCharCode + i);
        }
        currentCharCode = exhibitionCharCode;

        if (exhibition.locations) {
          exhibition.getLocations().then(function(locations) {
            exhibition.location = '';
            locations.forEach(function(location) {
              exhibition.location += location.name + ' ';
            });
          });
        }

        exhibs.push(exhibition);
      });

      //If names ended before Z, add everything up to Z
      for (var i = currentCharCode + 1; i <= 'Z'.charCodeAt(0); i++) {
        addLetter(i);
      }
    });

    $scope.getExhibitions = function() {
      letterHasMatch = {};
      var toplist = [];
      //customization START dgu2015: exhibitors on top

      //customization END dgu2015: exhibitors on top

      //Filter contacts by $scope.search.
      //Additionally, filter letters so that they only show if there
      //is one or more matching contact
      var list = exhibs.filter(function(item) {
        var itemDoesMatch = !$scope.search || item.isLetter || item.name.toLowerCase().indexOf($scope.search.toLowerCase()) > -1;

        //Mark this person's last name letter as 'has a match'
        if (!item.isLetter && itemDoesMatch) {
          var letter = item.name.charAt(0).toUpperCase();
          letterHasMatch[letter] = true;
        }

        return itemDoesMatch;
      }).filter(function(item) {
        //Finally, re-filter all of the letters and take out ones that don't
        //have a match
        if (item.isLetter && !letterHasMatch[item.letter]) {
          return false;
        }
        return true;
      });
      //customization START isvhld: topExhibitors always onTop
      // $scope.topExhibitors.concat(list);
      return $scope.topExhibitors.concat(list);
      //customization END isvhld: topExhibitors always onTop
    };

    //Letters are shorter, everything else is 52 pixels
    $scope.getItemHeight = function(item) {
      if (item) {
        var baseHeight = (item.logo && item.is_exhibition_list_logo_visible == 'true') ? 140 : 70;
        return item.isLetter ? 20 : (baseHeight + parseInt(item.name.length / 35) * 14);
      } else {
        return false;
      }
    };

    var letterHasMatch = {};

    $scope.clearSearch = function() {
      $scope.search = '';
    };

    $scope.headTitle = 'Exhibitors';

  }]);
})();
