(function() {
  'use strict';

  angular.module('favorites')
    .factory('$favorites', ['$common', '$database', function($common, $database,Notes) {
      var Favorite = $common.model();
      Favorite.tableName = 'favorites';
      Favorite.fields = {
        'reference_id': 'text primary key',
        'reference_type': 'integer',
        'created_at': 'integer',
        'modified_at': 'integer',
        'deleted_at': 'integer',
        'uploaded_at': 'text'
      };

      Favorite.getFavorite = function(id, type) {
        var query = 'Select * FROM favorites WHERE reference_id = ? AND reference_type = ?';
        return $database.getItem(query, [id, type]);
      };
      Favorite.getFavoritesByType = function(type) {
        var query = 'Select * FROM favorites WHERE reference_type = ? AND deleted_at is null';
        return $database.getItems(query, [type]);
      };
      Favorite.getNotes = function() {
        var query = 'Select * FROM notes WHERE deleted_at is null';
        // var query = 'Select notes.*, sessions.title as session,abstractpapers.title as abstract FROM notes Left JOIN sessions on notes.reference_id=sessions.id Left JOIN abstractpapers ON notes.reference_id=abstractpapers.id WHERE deleted_at is null';
        return $database.getItems(query, [],Notes);
      };
      Favorite.addFavorite = function(refId, refType) {
        var query = 'INSERT INTO favorites (reference_id,reference_type,deleted_at) VALUES (?,?,?)';
        return $database.execute(query, [refId, refType, null]);
      };
      Favorite.updateFavorite = function(refId, refType, deletedAt) {
        var query = 'UPDATE favorites SET deleted_at=? WHERE reference_id = ? AND reference_type = ?';
        return $database.execute(query, [deletedAt, refId, refType]);
      };
      Favorite.isFavorite = function(refId, refType) {
        var query = 'Select * FROM favorites  WHERE reference_id = ? AND reference_type = ? AND deleted_at is null';
        return $database.execute(query, [refId, refType]).then(function(res) {
          return ((res.rows.length === 1) ? true : false);
        });
      };


      Favorite.getElementsForModel = function(model, type) {
        var ids = [];
        return Favorite.getFavoritesByType(type).then(function(favorites) {
          favorites.forEach(function(entry) {
            ids.push(entry.reference_id);
          });

          return model.get(ids).then(function(elements) {
            return elements;
          });
        });
      };

      return Favorite;
    }]);
})();
