'use strict';

angular.module('ibeacon.controller', []).controller('BeaconsCtrl', function($scope, $beacons, $baseControllers) {
	$baseControllers.applyListController($scope, $beacons);

	$scope.beacons = [];
	var beacons = [];
	// Add your own beacon UUIDs to this list.
	var regions =
	[
		{uuid:'73676723-7400-0000-FFFF-0000FFFF0000'},
	];

	var locationManager = cordova.plugins.locationManager;

	$scope.startScan = function() {
		// The delegate object contains iBeacon callback functions.
		var delegate = locationManager.delegate.implement({
			didDetermineStateForRegion : function() {},

			didStartMonitoringForRegion : function() {},

			didRangeBeaconsInRegion : function(pluginResult) {
				
				for (var i in pluginResult.beacons) {
					// Insert beacon into table of found beacons.
					var beacon = pluginResult.beacons[i];
					beacon.timeStamp = Date.now();
					var key = beacon.uuid + ':' + beacon.major + ':' + beacon.minor;
					beacons[key] = beacon;
				}
			}
		});

		// Set the delegate object to use.
		locationManager.setDelegate(delegate);

		// Request permission from user to access location info.
		// This is needed on iOS 8.
		locationManager.requestAlwaysAuthorization();

		// Start monitoring and ranging beacons.
		for (var i in regions) {
			var beaconRegion = new locationManager.BeaconRegion(i + 1, regions[i].uuid);

			// Start monitoring.
			locationManager.startMonitoringForRegion(beaconRegion).fail(console.error).done();

			// Start ranging.
			locationManager.startRangingBeaconsInRegion(beaconRegion).fail(console.error).done();
		}
	};
	
	$scope.displayBeaconList = function ()
	{
		// Clear beacon list.
		$('#found-beacons').empty();

		var timeNow = Date.now();

		// Update beacon list.
		$.each(beacons, function(key, beacon)
		{
			// Only show beacons that are updated during the last 60 seconds.
			if (beacon.timeStamp + 60000 > timeNow)
			{
				// Valid RSSI values should be less than zero.
				var rssiWidth = Math.max(1, 100 + beacon.rssi);
				rssiWidth = Math.min(rssiWidth, 100);

				// Create tag to display beacon data.
				var element = $(
					'<li><strong>UUID: ' + beacon.uuid + '</strong><br />Major: ' + beacon.major + '<br />Minor: ' + beacon.minor + '<br />Proximity: ' + beacon.proximity + '<br />RSSI: ' + beacon.rssi + '<br /><div style="background:rgb(0,255,0);height:20px;width:'+rssiWidth + '%;"></div></li>'
				);

				$('#found-beacons').append(element);
			}
		});
	};

});
