'use strict';

angular.module('location.model', []).factory('$locations', function($common, $database,$exhibitions) {
	var Location = $common.model();
	Location.tableName = 'locations';
	Location.fields = {
		'id' : 'text primary key',
		'name' : 'text',
		'order' : 'integer',
		'short_name' : 'text',
		'location_type' : 'integer',
		'capacity' : 'integer'
	};

	Location.prototype.getMap = function() {
		var query = 'SELECT assigned_map as id,coordinate_x as lat,coordinate_y as lng,id AS markerid FROM locationlinks WHERE location = ? limit 1';
		return $database.getItems(query, [this.id]);
	};
	
	Location.prototype.getMapInfos = function(id) {
		var query = 'SELECT * FROM maps WHERE id = ? limit 1';
		return $database.getItems(query, [id]);
	};
	
	Location.prototype.getSessions = function() {
		var query = 'SELECT sessions.id, sessions.code, sessions.color, sessions.start_time, sessions.end_time, sessions.title, sessiontypes.name AS session_type FROM sessions JOIN sessiontypes ON sessiontypes.id = sessions.session_type WHERE sessions.locations LIKE "%' + this.id + '%" ORDER BY sessions.start_time, sessiontypes.\'order\', sessions.title';
		return $database.getItems(query, []);
	};

	Location.prototype.getExhibitions = function() {
		var query = 'SELECT * FROM exhibitions WHERE exhibitions.locations LIKE "%' + this.id + '%"';
		return $database.getItems(query, [], $exhibitions);
	};

	return Location;
}).factory('$locationproxys', function($common, $locations, $database) {
	var LocationProxy = new $common.model();
	LocationProxy.tableName = 'locationproxys';
	LocationProxy.fields = {
		'id' : 'text primary key',
		'linktype' : 'integer',
		'locations' : 'text'
	};

	LocationProxy.prototype.getLocations = function() {
		return $locations.get(this.locations.split(','));
	};

	LocationProxy.prototype.getLocationLinks = function() {
		var locationIds = this.locations.split(',').map(function(id) {
			return '"' + id + '"';
		}).join(','), query = 'SELECT locations.id, locations.name, locations.short_name, locations.location_type, locations.capacity, locationlinks.assigned_map, locationlinks.coordinate_x, locationlinks.coordinate_y FROM locations JOIN locationlinks ON locationlinks.location = locations.id WHERE locations.id IN (' + locationIds + ')';
		return $database.getItems(query, []);
	};

	return LocationProxy;
}).factory('$locationlinks', function($common) {
	var LocationLink = $common.model();
	LocationLink.tableName = 'locationlinks';
	LocationLink.fields = {
		'id' : 'text primary key',
		'location' : 'text',
		'assigned_map' : 'text',
		'coordinate_x' : 'integer',
		'coordinate_y' : 'integer',
		'Locations' : 'text',
		'message' : 'text',
		'name' : 'text',
		
	};

	return LocationLink;
});
