'use strict';

angular.module('maps.directives', []).directive('dmsMap', function() {
	return {
		restrict : 'E',
		scope : {
			id : '@',
			markerid : '@',
			centerlat : '@',
			centerlon : '@',
			startzoom : '@'
		},
		controller : function($scope, leafletBoundsHelpers, $timeout, $element, $compile, $maps) {
			var offsetWidth = $element.parent().prop('offsetWidth');
			var zoomingding = offsetWidth > 767 ? offsetWidth - 277 : offsetWidth;

			$scope.$watch('id', function() {
				if ($scope.id) {
					$scope.markers = {};
					var center_lat = $scope.centerlat || 0;
					var center_lon = $scope.centerlon || 0;
					var icons = {
						default_icon : {},
						info_icon : {
							iconUrl : 'img/maps/pins/pinGreen.png',
							iconSize : [32, 39],
							iconAnchor : [5, 32],
							popupAnchor : [2, -30]
						}
					};

					$maps.get($scope.id).then(function(map) {
						$scope.center = {
							lat : parseInt(center_lat),
							lng : parseInt(center_lon),
							zoom : (((map.width / zoomingding) * -1) + 1)
						};
						$scope.layers = {
							baselayers : {
								layer1 : {
									name : '',
									type : 'imageOverlay',
									url : map.map_file,
									bounds : [[0, 0], [map.height, map.width]],
									layerParams : {
										noWrap : true
									}
								}
							}

						};

						$scope.maxBounds = leafletBoundsHelpers.createBoundsFromArray([[200, -10], [map.height, map.width + 10]]);
						$scope.defaults = {
							scrollWheelZoom : false,
							zoomControl : false,
							crs : 'Simple',
							maxZoom : 1,
							minZoom : -3
						};

						$timeout(function() {
							$element.append($compile('<leaflet event-broadcast="events" maxbounds="maxBounds" center="center" layers="layers" defaults="defaults" markers="markierungenactive" height="100%" width="100%" style="background-color:#fff;"></leaflet>')($scope));
						}, 150);

					});

					$scope.events = {
						map : {
							enable : ['click'],
							logic : 'emit'
						}
					};

					var clickMapEvent = function(lat, lng, focus) {
						$scope.markierungenactive = {};
						var nearest = null;
						var i = 0;
						var k = 0;
						$scope.markers.forEach(function(marker) {
							var entfernung = ((marker.lat - lat) * (marker.lat - lat)) + ((marker.lng - lng) * (marker.lng - lng));
							if (nearest === null || entfernung < nearest) {
								nearest = entfernung;
								k = i;
							}
							i++;
						});
						$scope.markierungenactive[k] = $scope.markers[k];
						$timeout(function() {
							$scope.markierungenactive[k].focus = focus;
						}, 900);
					};

					$maps.getMarkers($scope.id).then(function(markers) {
						$scope.markers = markers;
						$scope.markers.forEach(function(markierung) {
							var markermessage = (markierung.orimessage || markierung.exhibition || markierung.name || 'Details');
							var markerRoomName = '';

							markierung.focus = false;
							markierung.draggable = false;
							markierung.clickable = true;
							markierung.opacity = 0.9;
							markierung.icon = icons.info_icon;
							markierung.message = '<a href="#app/locations/' + markierung.location + '">' + markermessage + '&nbsp;<span class="ion-ios-arrow-right pull-right" style="font-size:25px;margin-right:-10px; margin-top: -8px; color:white;"></span><br><span style="font-size:10px;color:white;">' + markerRoomName + '</span></a>';
							markierung.bounceOnAdd = true;
							markierung.bounceOnAddOptions = {
								duration : 900,
								height : 100
							};
						});
						$scope.$watch('markerid', function() {
							if ($scope.markerid) {
								$maps.getMarker($scope.markerid).then(function(marker) {
									$timeout(function() {
										clickMapEvent(marker.lat, marker.lng, false);
									}, 500);
								});
							}
						});
					});

					$scope.$on('leafletDirectiveMap.click', function(event, args) {
						clickMapEvent(args.leafletEvent.latlng.lat, args.leafletEvent.latlng.lng, true);
					});
				}
			});
		}
	};

});
