'use strict';

/**
 * @ngdoc directive
 * @name kongressapp.dms-notiz
 *
 *
 * @description
 * Hängt eine Notizenfunktion als element an jeden beliebigen Detailview.
 * @author: Michael Junge
 * @restrict E
 * @param {JUJU} id - Die ID zu der die Notiz gespeichert werden soll.
 * @param {Integer} type - Der Typ des Elements zu dem Das item gespeichert werden soll (sessions 1, presentation 2, persons 3, abstracts 4,exhibitions 10).
 *
 *
 **/

angular.module('note.directives', ['note.model']).directive('dmsNotesList', function() {
  var locations = {
    1: 'sessions',
    2: 'presentations',
    3: 'persons',
    4: 'abstractpapers',
    10: 'exhibitions'
  };
  return {
    restrict: 'E',
    transclude: true,
    scope: {
      data: '=',
    },
    link: function(scope) {

      scope.giveLocation = function(id) {
        return locations[id];
      };

      scope.$watch('data', function() {
        // scope.elements = scope.data;
        scope.elements = [];

        if (scope.data) {
          scope.data.forEach(function(element) {
            if (element.locations) {
              scope.elements.push(element);
            }
          });
        }
      });

    },
    templateUrl: 'modules/notes/views/list-directive.html'
  };
}).directive('dmsNotiz', function($translate) {
  return {
    restrict: 'E',
    transclude: false,
    scope: {
      id: '@',
      type: '@',
    },
    controller: function($scope, Notes, $timeout, $ionicScrollDelegate) {

      $scope.noteHint = '';
      $translate('NOTE_HINT').then(function(item) {
        $scope.noteHint = item;
      });

      /**
       * @ngdoc function
       * @name kongressapp.dms-notiz.getNotes
       * @description Holt alle notizen aus der Datenbank.
       */
      $scope.getNotes = function() {
        $scope.$watch('id', function() {
          if ($scope.id) {
            Notes.getNotes($scope.id, $scope.type).then(function(items) {
              $scope.items = angular.copy(items);
            });
          }
        });

        $scope.$watch('items', function() {
          if ($scope.items) {
            $scope.items.forEach(function(item) {
              item.edit = false;
            });
          }
        });
      };
      $scope.getNotes();
      /**
       * @ngdoc function
       * @name kongressapp.dms-notiz.addNote
       * @description Fügt eine Notiz hinzu. Diese wird nicht direkt gespeichert sondern nur angezeigt!
       */
      $scope.addNote = function() {
        $scope.items.push({
          content: '',
          edit: true,
          created_at: new Date().getTime(),
          modified_at: new Date().getTime(),
          reference_id: $scope.id,
          reference_type: $scope.type
        });
        $ionicScrollDelegate.scrollBottom();
        cordova.plugins.Keyboard.show();

        function keyboardHideHandler() {
          $ionicScrollDelegate.resize();
          $ionicScrollDelegate.scrollBottom();
        }


        window.addEventListener('native.keyboardhide', keyboardHideHandler);

      };
      /**
       * @ngdoc function
       * @name kongressapp.dms-notiz.saveNote
       * @description Speichert eine Notiz in der Datenbank ab.
       * @param {object} item - Das abzuspeichernde Item.
       *
       */
      $scope.saveNote = function(item) {
        Notes.saveNote(item);
        $timeout(function() {
          $scope.getNotes();
        }, 300);
      };
      /**
       * @ngdoc function
       * @name kongressapp.dms-notiz.deleteNote
       * @description Löscht eine Notiz aus der Datenbank (setzt deleted_at auf jetzt).
       * @param {object} item - Das zu löschende Item.
       *
       */
      $scope.deleteNote = function(item) {
        item.deleted_at = new Date().getTime();
        $scope.saveNote(item);
      };

    },
    templateUrl: 'modules/notes/views/note-directive.html'
  };
});
