'use strict';

angular.module('note.model', []).factory('Notes', function($common, $database) {
	var Note = $common.model();
	Note.tableName = 'notes';
	Note.fields = {
		'id' : 'integer primary key',
		'reference_id' : 'text',
		'reference_type' : 'integer',
		'content' : 'text',
		'created_at' : 'integer',
		'modified_at' : 'integer',
		'deleted_at' : 'integer',
	};

	Note.getNotes = function(id, type) {
		var query = 'Select id,reference_id,reference_type,content,modified_at FROM notes  WHERE reference_id = ? AND reference_type = ? AND deleted_at is null';
		return $database.getItems(query, [id, type]);
	};
	
	
	Note.getLinkedObject = function(ref_id,table, field){
	  var query = 'SELECT id, ' + field + ' from ' + table + ' WHERE id = ?';
	  return $database.getItem(query,[ref_id]); 
	};

	Note.saveNote = function(item) {
		var query;
		if (item.id) {
			query = 'UPDATE notes SET deleted_at=?,modified_at=?,content=? WHERE id = ?';
			return $database.execute(query, [item.deleted_at, new Date().getTime(), item.content, item.id]);
		} else {
			query = 'INSERT INTO notes (reference_id,reference_type,modified_at,created_at,content) VALUES (?,?,?,?,?)';
			return $database.execute(query, [item.reference_id, item.reference_type, new Date().getTime(), new Date().getTime(), item.content]);
		}
	};
	
	

	Note.getElementsForModel = function(model, type) {
		var ids = [];
		return Note.getNotesByType(type).then(function(notes) {
			notes.forEach(function(entry) {
				ids.push(entry.reference_id);
			});

			return model.get(ids).then(function(elements) {
				return elements;
			});
		});
	};

	return Note;
});