'use strict';

angular.module('persons.controller', []).controller('PersonsCtrl', function($scope, $ionicScrollDelegate, filterFilter, $persons, $filter, $translate) {
  var letters = $scope.letters = [];
  var contacts = $scope.contacts = [];
  var currentCharCode = 'A'.charCodeAt(0) - 1;

  $translate('SEARCH').then(function(item) {
    $scope.placeholder = item;
  });

  function addLetter(code) {
    var letter = String.fromCharCode(code);
    contacts.push({
      isLetter : true,
      letter : letter
    });
    letters.push(letter);
  }


  $persons.all().then(function(persons) {
    $scope.persons = $filter('orderBy')(persons, ['last_name', 'first_name']);

    $scope.persons.forEach(function(person) {
      //Get the first letter of the last name, and if the last name changes
      //put the letter in the array
      var personCharCode = person.last_name.toUpperCase().charCodeAt(0);
      //We may jump two letters, be sure to put both in
      //(eg if we jump from Adam Bradley to Bob Doe, add both C and D)
      var difference = personCharCode - currentCharCode;
      for (var i = 1; i <= difference; i++) {
        addLetter(currentCharCode + i);
      }
      currentCharCode = personCharCode;
      contacts.push(person);
    });

    //If names ended before Z, add everything up to Z
    for (var i = currentCharCode + 1; i <= 'Z'.charCodeAt(0); i++) {
      addLetter(i);
    }
  });

  $scope.getPersons = function() {
    letterHasMatch = {};
    //Filter contacts by $scope.search.
    //Additionally, filter letters so that they only show if there
    //is one or more matching contact
    return contacts.filter(function(item) {
      var itemDoesMatch = !$scope.search || item.isLetter || item.first_name.toLowerCase().indexOf($scope.search.toLowerCase()) > -1 || item.last_name.toLowerCase().indexOf($scope.search.toLowerCase()) > -1;

      //Mark this person's last name letter as 'has a match'
      if (!item.isLetter && itemDoesMatch) {
        var letter = item.last_name.charAt(0).toUpperCase();
        letterHasMatch[letter] = true;
      }

      return itemDoesMatch;
    }).filter(function(item) {
      //Finally, re-filter all of the letters and take out ones that don't
      //have a match
      if (item.isLetter && !letterHasMatch[item.letter]) {
        return false;
      }
      return true;
    });
  };

  //Letters are shorter, everything else is 52 pixels
  $scope.getItemHeight = function(item) {
    if (item) {
      return item.isLetter ? 20 : 60;
    }else{
      return false;
    }
  };

  $scope.scrollBottom = function() {
    $ionicScrollDelegate.scrollBottom(true);
  };

  var letterHasMatch = {};

  $scope.clearSearch = function() {
    $scope.search = '';
  };
}).controller('PersonDetailCtrl', function($scope, $stateParams, $persons, $database, $settings, Features) {
  $scope.isNative = $settings.isNative;
  $scope.platform = $settings.platform;
  $scope.shareEnabled = Features.shareFunction;
  var id = $stateParams.id;
  $persons.get(id).then(function(person) {

    person.getSessions().then(function(sessions) {
      $scope.sessions = sessions;
    });

    person.getPresentations().then(function(presentations) {
      $scope.presentations = presentations;
    });

    $scope.person = person;
  });

  $scope.share = function() {
    window.plugins.socialsharing.share('I am visiting ' + $scope.person.display_name);
  };
});

