'use strict';

angular.module('persons.model', []).factory('$persons', function($common, $database) {
  var Person = $common.model();
  Person.tableName = 'persons';
  Person.fields = {
    'id': 'text primary key',
    'form_of_address': 'text',
    'title': 'text',
    'first_name': 'text',
    'middle_name': 'text',
    'last_name': 'text',
    'vitae_text': 'text',
    'institution_1': 'text',
    'institution_2': 'text',
    'department': 'text',
    'street': 'text',
    'zip_code': 'text',
    'city': 'text',
    'state': 'text',
    'country': 'text',
    'phone': 'text',
    'fax': 'text',
    'email_1': 'text',
    'email_2': 'text',
    'vitae_address': 'text',
    'person_type': 'integer',
    'external_link': 'text',
    'vitae_picture': 'text',
    'display_name': 'text',
    'conference_user': 'text'
  };

  Person.prototype.getPresentations = function() {
    var query = 'SELECT schedualassignments.code, schedualassignments.start_time, schedualassignments.end_time, sessions.start_time AS sess_start, sessions.end_time AS sess_end,sessions.color, sessiontypes.name AS session_type, presentations.id, presentations.title, presentations.outline, roles.function FROM schedualassignments JOIN presentations ON schedualassignments.presentation = presentations.id JOIN sessions ON schedualassignments.session = sessions.id JOIN sessiontypes ON sessiontypes.id = sessions.session_type JOIN chairs ON presentations.chair = chairs.id JOIN roles ON roles.chair = chairs.id JOIN persons ON roles.person = persons.id WHERE persons.id = ? ORDER BY schedualassignments.start_time, sessiontypes.\'order\', presentations.title';
    return $database.getItems(query, [this.id]);
  };

  Person.prototype.getSessions = function() {
    var query = 'SELECT sessions.id, sessions.title, sessions.outline, sessions.code, sessiontypes.name AS session_type, sessions.color, sessions.font_color, sessions.type_group, sessions.start_time, sessions.end_time, sessions.chair, sessions.tags, sessions.locations, roles.function FROM sessions JOIN chairs ON sessions.chair = chairs.id JOIN roles ON roles.chair = chairs.id JOIN persons ON roles.person = persons.id JOIN sessiontypes ON sessiontypes.id = sessions.session_type WHERE persons.id = ? ORDER BY sessions.start_time, sessiontypes.\'order\', sessions.title';
    return $database.getItems(query, [this.id]);
  };

  return Person;
}).factory('$chairs', function($common, $database) {
  var Chair = $common.model();
  Chair.tableName = 'chairs';
  Chair.fields = {
    'id': 'text primary key',
    'persons': 'text'
  };

  Chair.relatedFields = {
    'persons': {
      'field': 'roles',
      'tableName': 'roles'
    }
  };

  Chair.prototype.getPersons = function() {
    var query = 'SELECT persons.id, persons.form_of_address, persons.title, persons.first_name, persons.middle_name, persons.last_name, persons.vitae_text, persons.institution_1, persons.institution_2, persons.department, persons.street, persons.zip_code, persons.city, persons.state, persons.country, persons.phone, persons.fax, persons.email_1, persons.email_2, persons.vitae_address, persons.person_type, persons.external_link, persons.vitae_picture, persons.display_name, roles.function FROM roles JOIN persons ON roles.person = persons.id AND roles.chair = ? ORDER BY roles.\'order\'';
    return $database.getItems(query, [this.id]);
  };

  return Chair;

}).factory('$rolemodel', function($common) {
  var Role = $common.model();
  Role.tableName = 'roles';
  Role.fields = {
    'id': 'text primary key',
    'person': 'text',
    'chair': 'text',
    'function': 'text',
    'order': 'integer'
  };

  Role.relatedFields = {
    'person': {
      'field': 'person',
      'tableName': 'persons',
    },
    'chair': {
      'field': 'chair',
      'tableName': 'chairs'
    }
  };

  return Role;
});
