'use strict';

angular.module('presentation.model', []).factory('$presentations', function($common, $database,$chairs) {
  var Presentation = $common.model();
  Presentation.tableName = 'presentations';
  Presentation.fields = {
    'id' : 'text primary key',
    'sessions' : 'text',
    'abstracts' : 'text',
    'title' : 'integer',
    'outline' : 'text',
    'external_link' : 'text',
    'chair' : 'integer'
  };

  Presentation.get = function(id) {
    var query = 'SELECT schedualassignments.code, sessiontypes.name AS session_type, schedualassignments.start_time, schedualassignments.end_time, presentations.* , sessions.color,sessions.font_color, persons.last_name, persons.first_name, persons.country  FROM schedualassignments JOIN presentations ON schedualassignments.presentation = presentations.id LEFT JOIN roles ON roles.chair = presentations.id LEFT JOIN persons ON persons.id = roles.person JOIN sessions on schedualassignments.session = sessions.id  JOIN sessiontypes ON sessiontypes.id = sessions.session_type ';

    if (Array.isArray(id)) {
      query += ' WHERE presentations.id IN (';
      var params = id.map(function() { return '?';});
      query += params.join(',') + ')';
      query += ' GROUP BY presentations.id ORDER BY schedualassignments.start_time, roles.\'order\'';
      return $database.getItems(query, id, Presentation);
    } else {
      query += ' WHERE presentations.id = ?';
      return $database.getItem(query, [id], Presentation);
    }
  };

  Presentation.prototype.getAbstracts = function() {
    var query = 'SELECT * FROM abstractpapers WHERE abstractpapers.id IN (' + this.abstracts.split(',').map(JSON.stringify).join(',') + ' ) AND (abstractpapers.embargo IS NULL OR abstractpapers.embargo < date(\'now\'))';
    return $database.getItems(query, []);
  };

  Presentation.prototype.getSessions = function() {
    var query = 'SELECT sessions.id, sessions.code, sessions.color,sessions.font_color, sessions.start_time, sessions.end_time, sessions.title, sessiontypes.name AS session_type FROM sessions JOIN sessiontypes ON sessiontypes.id = sessions.session_type JOIN schedualassignments ON sessions.id = schedualassignments.session WHERE schedualassignments.presentation = ?';
    return $database.getItems(query, [this.id]);
  };

  Presentation.prototype.getChair = function() {
    return $chairs.get(this.chair).then(function(chair) {
      return chair ? chair.getPersons() : [];
    });
  };

  Presentation.prototype.getCode = function() {
    var query = 'SELECT schedualassignments.code FROM schedualassignments WHERE schedualassignments.presentation = ?';
    return $database.getItem(query,[this.id]);
  };

  return Presentation;
});
