'use strict';
angular.module('push', ['ionic', 'ngCordova']).service('Push', function($settings, $cordovaPush, $rootScope, $cordovaDialogs, $http, $posts, $cordovaToast, $filter, $window, $update) {

  this.register = function() {
    var config = null;

    if (ionic.Platform.isAndroid()) {
      config = {
        'senderID' : $settings.androidPushSenderId // REPLACE THIS WITH YOURS FROM GCM CONSOLE - also in the project URL like: https://console.developers.google.com/project/434205989073
      };
    } else if (ionic.Platform.isIOS()) {
      config = {
        'badge' : 'true',
        'sound' : 'true',
        'alert' : 'true'
      };
    }

    $cordovaPush.register(config).then(function(result) {
      $rootScope.registerDisabled = true;
      $settings.pushActive = true;
      // ** NOTE: Android regid result comes back in the pushNotificationReceived, only iOS returned here
      if (ionic.Platform.isIOS()) {
        $rootScope.regId = result;
        storeDeviceToken('ios');
      }
    });
  };
  var updateMessages = function() {
    $update.syncAllResources();
  };

  // Android Notification Received Handler
  var handleAndroid = function(notification) {
    // ** NOTE: ** You could add code for when app is in foreground or not, or coming from coldstart here too
    //             via the console fields as shown.

    if (notification.event === 'registered') {
      $rootScope.regId = notification.regid;
      storeDeviceToken('android');
    } else if (notification.event === 'message') {
      $cordovaDialogs.alert(notification.message, $settings.pushTitle);
      updateMessages();
    }
  };

  // IOS Notification Received Handler
  var handleIOS = function(notification) {
    // Wenn die App aktiv ist
    if (notification.foreground === '1') {

      // Immer
      if (notification.body && notification.messageFrom) {
        $cordovaDialogs.alert(notification.body, notification.messageFrom);
        updateMessages();
      } else {
        $cordovaDialogs.alert(notification.alert, $settings.pushTitle);
        updateMessages();
      }
    } else {
      if (notification.badge) {
        $cordovaPush.setBadgeNumber(notification.badge);
      }
      // Immer
      if (notification.body && notification.messageFrom) {
        $cordovaDialogs.alert(notification.body, notification.messageFrom);
        updateMessages();
      } else {
        $cordovaDialogs.alert(notification.alert, $settings.pushTitle);
        updateMessages();
      }
    }
  };

  var storeDeviceToken = function() {
    var url = $settings.pushRegisterUrl;
    var platform;
    if (ionic.Platform.isIOS()) {
      platform = 'ios';
    } else if (ionic.Platform.isAndroid()) {
      platform = 'android';
    } else if (ionic.Platform.isWindowsPhone()) {
      platform = 'windows';
    }
    $window.localStorage.setItem('token', $rootScope.regId);
    var params = {
      'conference' : $settings.conferenceId,
      'type' : platform,
      'token' : $rootScope.regId
    };
    $http.post(url, params);

  };

  $rootScope.$on('pushNotificationReceived', function(event, notification) {
    updateMessages();
    if (ionic.Platform.isAndroid()) {
      handleAndroid(notification);
    } else if (ionic.Platform.isIOS()) {
      handleIOS(notification);
    }
  });
  // Dieser teil wird für iosbenutzt wie ist es bei android ??
  $rootScope.$on('$cordovaPush:notificationReceived', function(event, notification) {
    updateMessages();
    if (ionic.Platform.isAndroid()) {
      handleAndroid(notification);
    } else if (ionic.Platform.isIOS()) {
      handleIOS(notification);
    }
  });

});
