'use strict';

angular.module('sessions').controller('SessionGridCtrl', function($scope, $stateParams, $sessions, $filter, $ionicScrollDelegate) {
	var date = $stateParams.date;
	var rooms = $scope.rooms = {};
	var sessions = $scope.sessions = [];
	$scope.scaletop = {};
	$scope.scaleleft = {};
	// TODO: Zeiten dynamisch pro Tag auslesen
	var dayStart = 13,
	    dayEnd = 13,
	    pxPerMinute = 1.5,
	    width = 130;

	$scope.gridWidth = 0;
	$scope.gridHeight = 0;
	$scope.times = [];
	$scope.columnWidth = width;
	$scope.pxPerMinute = pxPerMinute;

	$sessions.roomsForDate(date).then(function(_rooms) {
		var roomPointer;
		var roomsCounter = 0;
		_rooms.forEach(function(room) {
			if (room.id !== roomPointer) {
				rooms[room.id] = {
					id : room.id,
					name : room.name,
					order : room.order,
					sessions : []
				};
				roomPointer = room.id;
				++roomsCounter;
			}
			sessions.push({
				id : room['session.id'],
				start_time : room['session.start_time'],
				end_time : room['session.end_time'],
				title : room['session.title'],
				code : room['session.code'],
				color : room['session.color'],
				font_color : room['session.font_color'],
				left : (roomsCounter - 1) * width,
				top : getOffsetForItem(room['session.start_time']),
				link : '#/app/sessions/' + room['session.id'],
			});

			var startHour = parseInt($filter('date')(room['session.start_time'], 'H'), 10);
			var endHour = parseInt($filter('date')(room['session.end_time'], 'H'), 10);
			dayStart = dayStart > startHour ? startHour : dayStart;
			dayEnd = dayEnd < endHour ? endHour : dayEnd;
		});
		sessions.map(function(session) {
			session.top = getOffsetForItem(session.start_time);
			return session;
		});
		$scope.gridWidth = width * roomsCounter + 100;
		$scope.times = getTimes(dayStart, dayEnd);
		$scope.gridHeight = ($scope.times.length + 1) * 30 * pxPerMinute;
	});

	$scope.positionHeaders = function() {
		$scope.scaleleft = {
			left : $ionicScrollDelegate.$getByHandle('mainScroll').getScrollPosition().left + 'px'
		};
		$scope.scaletop = {
			top : $ionicScrollDelegate.$getByHandle('mainScroll').getScrollPosition().top + 'px'
		};

		$scope.$apply();
	};

	$scope.calculateSizeForItem = function(item) {
		var hour_start = parseInt($filter('date')(item.start_time, 'H'), 10);
		var minute_start = parseInt($filter('date')(item.start_time, 'mm'), 10);
		var hour_end = parseInt($filter('date')(item.end_time, 'H'), 10);
		var minute_end = parseInt($filter('date')(item.end_time, 'mm'), 10);

		var start_time = (hour_start * 60) + minute_start;
		var end_time = (hour_end * 60) + minute_end;

		var result = end_time - start_time;
		var size = result;
		return size;
	};

	$scope.getSizeForItem = function(item) {
		var size = $scope.calculateSizeForItem(item) * pxPerMinute;
		return size + 'px';
	};

	$scope.fitsBox = function(item, size) {
		return $scope.calculateSizeForItem(item) >= size ? true : false;
	};

	var getOffsetForItem = function(time) {
		var hour_start = parseInt($filter('date')(time, 'H'), 10);
		var minute_start = parseInt($filter('date')(time, 'mm'), 10);
		var start_time = (hour_start * 60) + minute_start;

		var result = start_time - (dayStart * 60);

		return result * pxPerMinute;
	};

	$scope.getMaxContentHeight = function(item) {
		var max = $scope.calculateSizeForItem(item);
		return (max - 20) + 'px';
	};

	/**
	 * Die funktion ersetzt das eklige Zeiten Array.
	 *
	 *	@param	{int}	start	Die Startzeit.
	 *	@param	{int}	ende	Die Entzeit.
	 *	@return {Object}times	Gibt ein Array mit Zeiten in halbstunden zurück.
	 */
	var getTimes = function(start, ende) {
		var timesteps = (ende - start) * 2;
		var times = [];

		function pushtime() {
			times.push(start + ':30');
			start++;
		}

		for (var i = 0; i < timesteps + 1; i++) {
			if (i % 2 === 0) {
				times.push(start + ':00');
			} else {
				pushtime();
			}
		}
		console.log(times);
		return times;
	};

}).filter('orderObjectBy', function() {
	return function(items, field, reverse) {
		var filtered = [];
		angular.forEach(items, function(item) {
			filtered.push(item);
		});
		filtered.sort(function(a, b) {
			return (a[field] > b[field] ? 1 : -1);
		});
		if (reverse)
			filtered.reverse();
		return filtered;
	};
});

