(function() {
  'use strict';
  angular.module('sessions').factory('$sessions', ['$common', '$chairs', '$database', '$presentations',
  function($common, $chairs, $database, $presentations) {
    var Session = $common.model();
    Session.tableName = 'sessions';
    Session.fields = {
      'id' : 'text primary key',
      'code' : 'text',
      'session_type' : 'text',
      'color' : 'text',
      'font_color' : 'text',
      'type_group' : 'text',
      'start_time' : 'date',
      'end_time' : 'date',
      'locations' : 'text',
      'title' : 'title',
      'outline' : 'text',
      'external_link' : 'text',
      'chair' : 'text',
      'tags' : 'text',
      'poll' : 'text',
      'image' : 'text'
    };

    Session.relatedFields = {
      'presentations' : {
        'field' : 'id',
        'tableName' : 'schedualassignments',
        'relatedColoumn' : 'session'
      },
      'chair' : {
        'field' : 'chair',
        'tableName' : 'chairs'
      }
    };

    Session.now = function() {
      var query = 'SELECT sessions.id, sessions.code, sessions.color,sessions.font_color, sessions.start_time, sessions.end_time, sessions.title, sessiontypes.name AS session_type FROM sessions JOIN sessiontypes ON sessiontypes.id = sessions.session_type WHERE datetime(sessions.start_time) <= datetime(\'now\', \'localtime\') AND datetime(sessions.end_time) > datetime(\'now\', \'localtime\') ORDER BY sessions.start_time, sessiontypes.\'order\', sessions.title';
      return $database.getItems(query, []);
    };

    Session.date = function(date) {
      var query = 'SELECT sessions.id, sessions.code, sessions.color,sessions.font_color, sessions.start_time, sessions.end_time, sessions.title, sessiontypes.name AS session_type, sessions.image FROM sessions JOIN sessiontypes ON sessiontypes.id = sessions.session_type JOIN locations ON locations.id = sessions.locations WHERE start_time >= date(?) AND start_time <= date(?, \'+1 day\') ORDER BY start_time, locations.\'order\', sessions.title';
      return $database.getItems(query, [date, date], Session);
    };

    Session.roomsForDate = function(date) {
      var query = 'SELECT locations.id AS id, locations.name AS name, locations.\'order\' AS \'order\', sessions.id AS \'session.id\', sessions.start_time AS \'session.start_time\', sessions.color AS \'session.color\', sessions.font_color AS \'session.font_color\', sessions.end_time AS \'session.end_time\', sessions.title AS \'session.title\', sessions.code AS \'session.code\' FROM Locations JOIN sessions ON sessions.locations = locations.id WHERE start_time >= date(?) AND start_time <= date(?, \'+1 day\')  ORDER BY locations.\'order\' ';
      return $database.getItems(query, [date, date]);
    };

    Session.sessType = function() {
      var query = 'SELECT DISTINCT * FROM sessiontypes ORDER BY sessiontypes.\'order\'';
      return $database.getItems(query, []);
    };

    Session.sessTypeGroup = function() {
      var query = 'SELECT DISTINCT * FROM sessiontypegroups ORDER BY sessiontypegroups.\'order\'';
      return $database.getItems(query, []);
    };

    Session.sessionsByType = function(type) {
      var query = 'SELECT sessions.id, sessions.code, sessions.color,sessions.font_color,sessions.image, sessions.start_time, sessions.end_time, sessions.title, sessiontypes.name AS session_type FROM sessions JOIN sessiontypes ON sessiontypes.id = sessions.session_type WHERE sessions.session_type = ? ORDER BY sessions.start_time, sessions.title';
      return $database.getItems(query, [type], Session);
    };

    Session.sessionsByTypeGroup = function(typegroup) {
      var query = 'SELECT sessions.id, sessions.code, sessions.color,sessions.font_color,sessions.image, sessions.start_time, sessions.end_time, sessions.title, sessiontypegroups.name AS session_type_group FROM sessions JOIN sessiontypegroups ON sessiontypegroups.id = sessions.session_type_group WHERE sessions.session_type_group = ? ORDER BY sessions.start_time, sessions.title';
      return $database.getItems(query, [typegroup], Session);
    };

    Session.getByTag = function(tag) {
      var query = 'SELECT sessions.id, sessions.code, sessions.color,sessions.image, sessions.start_time, sessions.end_time, sessions.title, sessiontypes.name AS session_type FROM sessions JOIN sessiontypes ON sessiontypes.id = sessions.session_type WHERE sessions.tags LIKE ? ORDER BY sessions.start_time, sessions.title';
      console.log($database.getItems(query, ['%' + tag + '%'], Session));
      return $database.getItems(query, ['%' + tag + '%'], Session);
    };

    Session.get = function(id) {
      var query = 'SELECT sessions.id, sessions.outline, sessions.locations, sessions.outline, sessions.external_link, sessions.chair, sessions.poll, sessions.code, sessions.color,sessions.font_color, sessions.start_time, sessions.end_time, sessions.title, sessiontypes.name AS session_type FROM sessions JOIN sessiontypes ON sessiontypes.id = sessions.session_type';

      if (Array.isArray(id)) {
        query += ' WHERE sessions.id IN (';
        var params = id.map(function() {
          return '?';
        });
        query += params.join(',') + ')';
        return $database.getItems(query, id, Session);
      } else {
        query += ' WHERE sessions.id = ?';
        return $database.getItem(query, [id], Session);
      }
    };

    Session.prototype.getChair = function() {
      return $chairs.get(this.chair).then(function(chair) {
        return chair ? chair.getPersons() : [];
      });
    };

    Session.prototype.getPresentations = function() {
      var query = 'SELECT schedualassignments.id AS linkId, schedualassignments.code, schedualassignments.start_time, schedualassignments.end_time, presentations.id, sessions.color,sessions.font_color, sessiontypes.name AS session_type, presentations.abstracts, presentations.title, presentations.outline, presentations.external_link, persons.last_name, persons.first_name,persons.city, persons.country, roles.\'order\' as roleOrder  FROM schedualassignments JOIN presentations ON schedualassignments.presentation = presentations.id JOIN sessions on schedualassignments.session = sessions.id  JOIN sessiontypes ON sessiontypes.id = sessions.session_type LEFT JOIN roles ON roles.chair = presentations.id LEFT JOIN persons ON persons.id = roles.person WHERE schedualassignments.session = ? ORDER BY schedualassignments.start_time, schedualassignments.code, roles.\'order\'';
      return $database.getItems(query, [this.id], $presentations).then(function(presentations) {
        presentations = presentations.reduce(function(newPresentations, presentation) {
          var speaker = {
            first_name : presentation.first_name,
            last_name : presentation.last_name,
            country : presentation.country,
            city : presentation.city,
            order : presentation.roleOrder,
          };
          if (!newPresentations[presentation.id]) {
            newPresentations[presentation.id] = presentation;
            newPresentations[presentation.id].speakers = [];
          }
          newPresentations[presentation.id].speakers.push(speaker);
          return newPresentations;
        }, {});

        return Object.keys(presentations).map(function(k) {
          return presentations[k];
        });
      });
    };

    // TODO DEPRECATED
    Session.prototype.getTags = function() {
      var query = 'SELECT tags.id, tags.title, tags.color FROM tags WHERE tags.id IN (' + this.tags.split(',').map(JSON.stringify).join(',') + ' )';
      return $database.getItems(query, []);
    };

    Session.prototype.getLocations = function() {
      var query = 'SELECT * FROM locations WHERE locations.id IN (' + this.locations.split(',').map(JSON.stringify).join(',') + ' )';
      return $database.getItems(query, []);
    };
    Session.prototype.getPoll = function(poll) {
      var query = 'SELECT polls.title,state.ready FROM polls JOIN (SELECT count(answer) as ready FROM votes Where poll = ? AND question = "" AND votes.reference_id = ?) as state WHERE polls.id = ?';
      return $database.getItem(query, [poll, this.id, poll]);
    };

    return Session;
  }]);
})();
