'use strict';

angular.module('sync', ['ionic', 'ngCordova', 'sync.upload', 'sync.socket', 'sync.files']).factory('$update', function($settings, $database, $abstractpapers, $sessions, $menu, $exhibitions, $generalinfos, $locations, $maps, $persons, $chairs, $presentations, Tags, $favorites, $sessionpresentationlinks, $rolemodel, $locationlinks, $posts, Messages, $chatcontacts, Sponsors, Sponsorlevels, Polls, Questions, Votes, Notes, Medias, $pubsub, SessionTypes, AnalyticEvent, Socket, $timeout) {

  var resourceEndpointUrl = $settings.apiHost + '/conference/' + $settings.conferenceId + '/';
  var resources = {};

  resources[resourceEndpointUrl + 'sessiontypes/'] = SessionTypes;
  resources[resourceEndpointUrl + 'abstracts/'] = $abstractpapers;
  resources[resourceEndpointUrl + 'exhibitions/'] = $exhibitions;
  resources[resourceEndpointUrl + 'generalinformations/'] = $generalinfos;
  resources[resourceEndpointUrl + 'locations/'] = $locations;
  resources[resourceEndpointUrl + 'locationlinks/'] = $locationlinks;
  resources[resourceEndpointUrl + 'maps/'] = $maps;
  resources[resourceEndpointUrl + 'menulinks/'] = $menu;
  resources[resourceEndpointUrl + 'persons/'] = $persons;
  resources[resourceEndpointUrl + 'chairs/'] = $chairs;
  resources[resourceEndpointUrl + 'presentations/'] = $presentations;
  resources[resourceEndpointUrl + 'sessions/'] = $sessions;
  resources[resourceEndpointUrl + 'tags/'] = Tags;
  resources[resourceEndpointUrl + 'favorites/'] = $favorites;
  resources[resourceEndpointUrl + 'sessionpresentationlinks/'] = $sessionpresentationlinks;
  resources[resourceEndpointUrl + 'roles/'] = $rolemodel;
  resources[resourceEndpointUrl + 'news/'] = $posts;
  resources[resourceEndpointUrl + 'chat/messages/'] = Messages;
  resources[resourceEndpointUrl + 'sponsors/'] = Sponsors;
  resources[resourceEndpointUrl + 'sponsorlevels/'] = Sponsorlevels;
  resources[resourceEndpointUrl + 'polls/'] = Polls;
  resources[resourceEndpointUrl + 'questions/'] = Questions;
  resources[resourceEndpointUrl + 'medias/'] = Medias;
  var init = function() {
    $database.createTableForModel($chatcontacts);
    $database.createTableForModel(Votes);
    $database.createTableForModel(Notes);
    $database.createTableForModel(AnalyticEvent);
    $database.execute('CREATE TABLE IF NOT EXISTS timestamps (id integer primary key, tablename text, timestamp text)');

    Socket.join('registerForDatasync:' + $settings.conferenceId);
    Socket.on('dataSyncUpdate', syncAllResources);

  };

  var syncAllResources = function() {
    for (var url in resources) {
      fetchResource(resources[url], url);
    }
  };

  var syncAllResourcesInterval = function() {
    syncAllResources();
    $timeout(function() {
      syncAllResourcesInterval();
    }, 300000);
  };

  var fetchResource = function(modelClass, resourceUrl) {
    $database.createTableForModel(modelClass).then(function() {
      $database.syncResource(modelClass, resourceUrl);
    });
  };

  var syncPersonalResources = function() {
    fetchResource($favorites, resourceEndpointUrl + 'favorites/');
    fetchResource(Messages, resourceEndpointUrl + 'chat/messages/');

  };

  $pubsub.subscribe('logged_in', syncPersonalResources);

  return {
    syncAllResources : syncAllResources,

    fetchResource : fetchResource,

    init : init,

    syncAllResourcesInterval : syncAllResourcesInterval,
  };
}).run(function($ionicPlatform, $state, $timeout, $update,$ionicSideMenuDelegate) {
  $ionicPlatform.ready(function() {
    document.addEventListener("deviceReady", function() {
      $update.syncAllResourcesInterval();
      document.addEventListener("resume", function() {
        $timeout(function() {
          $update.syncAllResources();
          if(!$ionicSideMenuDelegate.isOpenLeft()){
          	
	          $ionicSideMenuDelegate.toggleLeft();
          }
        }, 0);
      }, false);
    });
  });
});
