'use strict';

/**
 * Start der Applikation
 *
 * angular.module is a global place for creating, registering and retrieving Angular modules
 * 'start' is the name of this angular module example (also set in a <body> attribute in index.html)
 * the 2nd parameter is an array of 'requires'
 */
angular.module('start', ['ionic', 'ngCordova', 'settings', 'features', 'submodules', 'subdirectives', 'database', 'sync', 'leaflet-directive', 'ngFitText', 'simplePagination']).run(function($ionicPlatform, $rootScope, $settings) {
  $ionicPlatform.ready(function() {
    $settings.isNative = ionic.Platform.isWebView();

    if (ionic.Platform.isIOS()) {
      $settings.platform = 'ios';
    } else {
      $settings.platform = '';
    }
    // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
    // for form inputs)
    if (window.cordova && window.cordova.plugins.Keyboard) {
      cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
    }
    if (window.StatusBar) {
      // org.apache.cordova.statusbar required
      if ($settings.blackBar) {
        StatusBar.styleDefault();
      }
    }
    $rootScope.$apply();
  });

  // Erste Schritte mit dem update via SQLite
}).run(function($ionicPlatform, $update, Upload, $settings, $http, User, $cordovaPush, $rootScope, $cordovaDialogs, $timeout, Socket, FileUpdate, $ionicHistory, $ionicPopup, $window,$ionicSideMenuDelegate) {

  $ionicPlatform.ready(function() {
    if (ionic.Platform.isIOS() || ionic.Platform.isAndroid()) {
      $timeout(function() {
        navigator.splashscreen.hide();
		$ionicSideMenuDelegate.toggleLeft();
      }, $settings.splashTime);
    }
    var onOnline = function() {
      // is needed to handle wanky online events
      document.removeEventListener('online', onOnline, false);
      $timeout(function() {
        document.addEventListener('online', onOnline, false);
      }, 30000);

      Upload.uploadResources();
      $update.init();
      $update.syncAllResources();
      Upload.uploadResources();
    };

    FileUpdate.init();
    document.addEventListener('online', onOnline, false);

    if (ionic.Platform.isAndroid()) {
      $ionicPlatform.registerBackButtonAction(function(e) {
        if ($ionicHistory.backView() !== null) {
          $ionicHistory.goBack();
        } else {
          var confirmPopup = $ionicPopup.confirm({
            title : 'Confirm Exit',
            template : 'Wollen Sie die App wirklich beenden?'
          });
          confirmPopup.then(function(close) {
            if (close) {
              // there is no back view, so close the app instead
              ionic.Platform.exitApp();
            }// otherwise do nothing
            console.log("User canceled exit.");
          });
        }

        e.preventDefault();
        return false;
      }, 101);
    }

  });

}).run(function($ionicPlatform, $update, $settings, $http, User, Push, Upload, $rootScope, $location) {
  $ionicPlatform.ready(function() {
    if (User.secret) {
      $http.defaults.headers.common.Authorization = 'Token ' + User.secret;
    }

    $settings.pushActive = true;

    if (ionic.Platform.isWebView()) {
      Push.register();
    }

    $update.init();
    $update.syncAllResources();
    Upload.uploadResources();
  });

}).config(function($stateProvider, $urlRouterProvider) {

  // Wenn kein korrekter Provider gefunden wurde und die Startseite.
  $urlRouterProvider.otherwise('/app/sessions/date/2015-09-23');
}).config(['$compileProvider',
function($compileProvider) {
  $compileProvider.imgSrcSanitizationWhitelist(/^\s*(https?|file|blob|cdvfile):|data:image\//);
}]).config(['$httpProvider',
function($httpProvider) {
  $httpProvider.defaults.useXDomain = true;
  delete $httpProvider.defaults.headers.common['X-Requested-With'];
}]);
